
/* 
 * Copyright (c) 1993-2003,2012 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*------------------  Fichier gnr automatiquement ------------------*/
/*------------------  Ne pas diter manuellement !!! ------------------*/

#ifndef rmp400_MSG_LIB_H
#define rmp400_MSG_LIB_H

#include "portLib.h"
#include "csLib.h"
#include "posterLib.h"
 
#include "rmp400Type.h"

#define TIME_WAIT_REPLY           /*2000*/ 0
 



/*----------- Tailles des messages et boites aux lettres  ----------*/

/* Taille du buffer utilise dans la bal pour un mesage de taille size
 * (cf servMbox pour UNIX et h2rgnLib pour VXWORKS )
 */


#define BUF_SIZE(size) \
   (size + 4 - (size & 3) + sizeof(LETTER_HDR) + 8)

/**
 ** Boite aux lettres reception requetes 
 **/
#define  RMP400_MBOX_NAME                  genomInstanceName("rmp400")

#define  RMP400_MAX_RQST_SIZE              128

#define  RMP400_MBOX_RQST_SIZE       \
 (BUF_SIZE(RMP400_MAX_RQST_SIZE) * SERV_NMAX_RQST_ID)

/** 
 ** Taille max repliques (serveur)
 **/
#define  RMP400_MAX_INTERMED_REPLY_SIZE   (sizeof(int))

#define  RMP400_MAX_REPLY_SIZE             56

/** 
 ** Taille bal clients pour reception repliques 
 **/
#define  RMP400_CLIENT_MBOX_REPLY_SIZE   \
((BUF_SIZE(RMP400_MAX_REPLY_SIZE) \
   + BUF_SIZE(RMP400_MAX_INTERMED_REPLY_SIZE)) \
 * CLIENT_NMAX_RQST_ID)

/*-------------------------- Liste des requetes  -------------------------*/

#define RMP400_INIT_RQST 0
#define RMP400_SETCMDPARAM_RQST 1
#define RMP400_SETKINEMATICSPARAM_RQST 2
#define RMP400_SETVARPARAM_RQST 3
#define RMP400_STOP_RQST 4
#define RMP400_JOYSTICKON_RQST 5
#define RMP400_TRACK_RQST 6
#define RMP400_SETPOM_RQST 7
#define RMP400_SETPOS_RQST 8
#define RMP400_GYRO_RQST 9
#define RMP400_GETGYRO_RQST 10
#define RMP400_SETODOMETRYMETHOD_RQST 11
#define RMP400_SETWHEELSIZECORRECTION_RQST 12
#define RMP400_LOG_RQST 13
#define RMP400_STOPLOG_RQST 14
#define RMP400_LOGINTERNALDATA_RQST 15
#define RMP400_STOPINTERNALDATALOG_RQST 16
#define RMP400_GOTO_RQST 17
#define RMP400_ABORT_RQST 18


/*---------------- PROTOTYPES DES FONCTIONS EXTERNES ------------------*/

#ifdef __cplusplus
extern "C" {
#endif

STATUS rmp400ClientInit (CLIENT_ID *pClientId);

STATUS rmp400ClientEnd (CLIENT_ID clientId);

int rmp400AbortRqstSend (CLIENT_ID clientId, 
			   int *pRqstId,
			   int *activity,
			   int replyTimeOut);

int rmp400AbortReplyRcv (CLIENT_ID clientId, 
			   int rqstId, 
			   int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			   int *bilan);

int rmp400AbortRqstAndRcv (CLIENT_ID clientId, 
			     int *activity,
			     int *bilan);




extern STATUS rmp400InitRqstSend (CLIENT_ID clientId, int *pRqstId,
					 
					 int replyTimeOut);

extern int rmp400InitReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int rmp400InitRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					 
					
					int *activity, int *bilan);

extern int rmp400InitRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					 
					
					int *activity,
					int *bilan);




extern STATUS rmp400SetCmdParamRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 GENPOS_CART_CMD_PARAM *(in_cmd),
					 int replyTimeOut);

int rmp400SetCmdParamReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int rmp400SetCmdParamRqstAndRcv (CLIENT_ID clientId, 
				 GENPOS_CART_CMD_PARAM *(in_cmd), 
				 
				 int *bilan);




extern STATUS rmp400SetKinematicsParamRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 RMP400_KINEMATICS_STR *(in_kinematics),
					 int replyTimeOut);

int rmp400SetKinematicsParamReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int rmp400SetKinematicsParamRqstAndRcv (CLIENT_ID clientId, 
				 RMP400_KINEMATICS_STR *(in_kinematics), 
				 
				 int *bilan);




extern STATUS rmp400SetVarParamRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 RMP400_VAR_PARAMS *(in_varParams),
					 int replyTimeOut);

int rmp400SetVarParamReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int rmp400SetVarParamRqstAndRcv (CLIENT_ID clientId, 
				 RMP400_VAR_PARAMS *(in_varParams), 
				 
				 int *bilan);




extern STATUS rmp400StopRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int rmp400StopReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int rmp400StopRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS rmp400JoystickOnRqstSend (CLIENT_ID clientId, int *pRqstId,
					 
					 int replyTimeOut);

extern int rmp400JoystickOnReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int rmp400JoystickOnRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					 
					
					int *activity, int *bilan);

extern int rmp400JoystickOnRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					 
					
					int *activity,
					int *bilan);




extern STATUS rmp400TrackRqstSend (CLIENT_ID clientId, int *pRqstId,
					 RMP400_TRACK_PARAMS *(in_trackParams),
					 int replyTimeOut);

extern int rmp400TrackReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int rmp400TrackRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					RMP400_TRACK_PARAMS *(in_trackParams), 
					
					int *activity, int *bilan);

extern int rmp400TrackRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					RMP400_TRACK_PARAMS *(in_trackParams), 
					
					int *activity,
					int *bilan);




extern STATUS rmp400SetPomRqstSend (CLIENT_ID clientId, int *pRqstId,
					 GENPOS_POSTER_NAME *(in_mePosterName),
					 int replyTimeOut);

extern int rmp400SetPomReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int rmp400SetPomRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					GENPOS_POSTER_NAME *(in_mePosterName), 
					
					int *activity, int *bilan);

extern int rmp400SetPomRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					GENPOS_POSTER_NAME *(in_mePosterName), 
					
					int *activity,
					int *bilan);




extern STATUS rmp400SetPosRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 RMP400_CART_3DCONFIG *(in_setPosStr),
					 int replyTimeOut);

int rmp400SetPosReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int rmp400SetPosRqstAndRcv (CLIENT_ID clientId, 
				 RMP400_CART_3DCONFIG *(in_setPosStr), 
				 
				 int *bilan);




extern STATUS rmp400GyroRqstSend (CLIENT_ID clientId, int *pRqstId,
					 RMP400_GYRO_PARAMS *(in_params),
					 int replyTimeOut);

extern int rmp400GyroReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int rmp400GyroRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					RMP400_GYRO_PARAMS *(in_params), 
					
					int *activity, int *bilan);

extern int rmp400GyroRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					RMP400_GYRO_PARAMS *(in_params), 
					
					int *activity,
					int *bilan);




extern STATUS rmp400GetGyroRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int rmp400GetGyroReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       RMP400_GYRO_PARAMS *(out_params),
			       int *bilan);

int rmp400GetGyroRqstAndRcv (CLIENT_ID clientId, 
				  
				 RMP400_GYRO_PARAMS *(out_params),
				 int *bilan);




extern STATUS rmp400SetOdometryMethodRqstSend (CLIENT_ID clientId, int *pRqstId,
					 RMP400_ODOMETRY_METHOD *(in_odometryCommand),
					 int replyTimeOut);

extern int rmp400SetOdometryMethodReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int rmp400SetOdometryMethodRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					RMP400_ODOMETRY_METHOD *(in_odometryCommand), 
					
					int *activity, int *bilan);

extern int rmp400SetOdometryMethodRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					RMP400_ODOMETRY_METHOD *(in_odometryCommand), 
					
					int *activity,
					int *bilan);




extern STATUS rmp400SetWheelSizeCorrectionRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 double *(in_wheelSizeCorrection),
					 int replyTimeOut);

int rmp400SetWheelSizeCorrectionReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int rmp400SetWheelSizeCorrectionRqstAndRcv (CLIENT_ID clientId, 
				 double *(in_wheelSizeCorrection), 
				 
				 int *bilan);




extern STATUS rmp400LogRqstSend (CLIENT_ID clientId, int *pRqstId,
					 GEN_STRING128 *(in_fileName),
					 int replyTimeOut);

extern int rmp400LogReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int rmp400LogRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					GEN_STRING128 *(in_fileName), 
					
					int *activity, int *bilan);

extern int rmp400LogRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					GEN_STRING128 *(in_fileName), 
					
					int *activity,
					int *bilan);




extern STATUS rmp400StopLogRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int rmp400StopLogReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int rmp400StopLogRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS rmp400LogInternalDataRqstSend (CLIENT_ID clientId, int *pRqstId,
					 GEN_STRING128 *(in_fileName),
					 int replyTimeOut);

extern int rmp400LogInternalDataReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int rmp400LogInternalDataRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					GEN_STRING128 *(in_fileName), 
					
					int *activity, int *bilan);

extern int rmp400LogInternalDataRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					GEN_STRING128 *(in_fileName), 
					
					int *activity,
					int *bilan);




extern STATUS rmp400StopInternalDataLogRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int rmp400StopInternalDataLogReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int rmp400StopInternalDataLogRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS rmp400GotoRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 GENPOS_CART_REF *(in_ref),
					 int replyTimeOut);

int rmp400GotoReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int rmp400GotoRqstAndRcv (CLIENT_ID clientId, 
				 GENPOS_CART_REF *(in_ref), 
				 
				 int *bilan);


#ifdef __cplusplus
}
#endif
#endif /* rmp400_MSG_LIB_H */

